@@ -28,7 +28,8 @@ import Network.Wai (Request(..), Middleware, responseStatus, Response, responseH
 import System.Log.FastLogger
 import Network.HTTP.Types as H
 import Data.Maybe (fromMaybe)
-import Data.Monoid (mconcat)
+import Data.Monoid (mconcat, (<>))
+import Data.Time (getCurrentTime, diffUTCTime)
 
 import Network.Wai.Parse (sinkRequestBody, lbsBackEnd, fileName, Param, File, getRequestBodyType)
 import qualified Data.ByteString.Lazy as LBS
@@ -56,6 +57,11 @@ data Destination = Handle Handle
 
 type Callback = LogStr -> IO ()
 
+-- | @RequestLoggerSettings@ is an instance of Default. See <https://hackage.haskell.org/package/data-default Data.Default> for more information.
+--
+-- @outputFormat@, @autoFlush@, and @destination@ are record fields
+-- for the record type @RequestLoggerSettings@, so they can be used to
+-- modify settings values using record syntax.
 data RequestLoggerSettings = RequestLoggerSettings
     {
       -- | Default value: @Detailed@ @True@.
@@ -112,32 +118,18 @@ customMiddleware cb getdate formatter app req sendResponse = app req $ \res -> d
     sendResponse res
 
 -- | Production request logger middleware.
--- Implemented on top of "logCallback", but prints to 'stdout'
+{-# NOINLINE logStdout #-}
 logStdout :: Middleware
 logStdout = unsafePerformIO $ mkRequestLogger def { outputFormat = Apache FromSocket }
 
 -- | Development request logger middleware.
--- Implemented on top of "logCallbackDev", but prints to 'stdout'
 --
 -- Flushes 'stdout' on each request, which would be inefficient in production use.
 -- Use "logStdout" in production.
+{-# NOINLINE logStdoutDev #-}
 logStdoutDev :: Middleware
 logStdoutDev = unsafePerformIO $ mkRequestLogger def
 
--- no black or white which are expected to be existing terminal colors.
-colors0 :: [Color]
-colors0 = [
-    Green
-  , Yellow
-  , Blue
-  , Magenta
-  , Cyan
-  ]
-
-rotateColors :: [Color] -> ([Color], Color)
-rotateColors [] = error "Impossible! There must be colors!"
-rotateColors (c:cs) = (cs ++ [c], c)
-
 -- | Prints a message using the given callback function for each request.
 -- This is not for serious production use- it is inefficient.
 -- It immediately consumes a POST body and fills it back in and is otherwise inefficient
@@ -146,44 +138,60 @@ rotateColors (c:cs) = (cs ++ [c], c)
 -- This meanst that you can accurately see the interleaving of requests.
 -- And if the app crashes you have still logged the request.
 -- However, if you are simulating 10 simultaneous users you may find this confusing.
--- The request and response are connected by color on Unix and also by the request path.
 --
 -- This is lower-level - use 'logStdoutDev' unless you need greater control.
 --
 -- Example ouput:
 --
 -- > GET search
--- > Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8
--- >
--- > Status: 200 OK. search
+-- >   Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8
+-- >   Status: 200 OK 0.010555s
 -- >
 -- > GET static/css/normalize.css
--- > Accept: text/css,*/*;q=0.1
--- > GET [("LXwioiBG","")]
--- >
--- > Status: 304 Not Modified. static/css/normalize.css
+-- >   Params: [("LXwioiBG","")]
+-- >   Accept: text/css,*/*;q=0.1
+-- >   Status: 304 Not Modified 0.010555s
 
 detailedMiddleware :: Callback -> Bool -> IO Middleware
-detailedMiddleware cb useColors = do
-    getAddColor <-
-        if not useColors then return (return return) else do
-                icolors <- newIORef colors0
-                return $ do
-                    color <- liftIO $ atomicModifyIORef icolors rotateColors
-                    return $ ansiColor color
-    return $ detailedMiddleware' cb getAddColor
+detailedMiddleware cb useColors =
+    let (ansiColor, ansiMethod, ansiStatusCode) =
+          if useColors
+            then (ansiColor', ansiMethod', ansiStatusCode')
+            else (\_ t -> [t], (:[]), \_ t -> [t])
 
-ansiColor :: Color -> BS.ByteString -> [BS.ByteString]
-ansiColor color bs = [
-    pack $ setSGRCode [SetColor Foreground Vivid color]
-  , bs
-  , pack $ setSGRCode [Reset]
-  ]
+    in return $ detailedMiddleware' cb ansiColor ansiMethod ansiStatusCode
+
+ansiColor' :: Color -> BS.ByteString -> [BS.ByteString]
+ansiColor' color bs =
+    [ pack $ setSGRCode [SetColor Foreground Dull color]
+    , bs
+    , pack $ setSGRCode [Reset]
+    ]
+
+-- | Tags http method with a unique color.
+ansiMethod' :: BS.ByteString -> [BS.ByteString]
+ansiMethod' m = case m of
+    "GET"    -> ansiColor' Cyan m
+    "HEAD"   -> ansiColor' Cyan m
+    "PUT"    -> ansiColor' Green m
+    "POST"   -> ansiColor' Yellow m
+    "DELETE" -> ansiColor' Red m
+    _        -> ansiColor' Magenta m
+
+ansiStatusCode' :: BS.ByteString -> BS.ByteString -> [BS.ByteString]
+ansiStatusCode' c t = case S8.take 1 c of
+    "2"     -> ansiColor' Green t
+    "3"     -> ansiColor' Yellow t
+    "4"     -> ansiColor' Red t
+    "5"     -> ansiColor' Magenta t
+    _       -> ansiColor' Blue t
 
 detailedMiddleware' :: Callback
-                    -> IO (BS.ByteString -> [BS.ByteString])
+                    -> (Color -> BS.ByteString -> [BS.ByteString])
+                    -> (BS.ByteString -> [BS.ByteString])
+                    -> (BS.ByteString -> BS.ByteString -> [BS.ByteString])
                     -> Middleware
-detailedMiddleware' cb getAddColor app req sendResponse = do
+detailedMiddleware' cb ansiColor ansiMethod ansiStatusCode app req sendResponse = do
     let mlen = lookup "content-length" (requestHeaders req) >>= readInt
     (req', body) <-
         case mlen of
@@ -215,49 +223,39 @@ detailedMiddleware' cb getAddColor app req sendResponse = do
             _ -> return (req, [])
 
     postParams <- if requestMethod req `elem` ["GET", "HEAD"]
-      then return []
-      else do postParams <- liftIO $ allPostParams body
-              return $ collectPostParams postParams
+        then return []
+        else do postParams <- liftIO $ allPostParams body
+                return $ collectPostParams postParams
 
     let getParams = map emptyGetParam $ queryString req
+        accept = fromMaybe "" $ lookup "Accept" $ requestHeaders req
+        params = let par | not $ null postParams = [pack (show postParams)]
+                         | not $ null getParams  = [pack (show getParams)]
+                         | otherwise             = []
+                 in if null par then [""] else ansiColor White "  Params: " <> par <> ["\n"]
 
-    addColor <- getAddColor
-    let accept = fromMaybe "" $ lookup "Accept" $ requestHeaders req
-
-    -- log the request immediately.
-    liftIO $ cb $ mconcat $ map toLogStr $ addColor (requestMethod req) ++
-        [ " "
-        , rawPathInfo req
-        , " :: "
-        , accept
-        , paramsToBS  "GET " getParams
-        , paramsToBS "POST " postParams
-        , "\n"
-        ]
-
+    t0 <- getCurrentTime
     app req' $ \rsp -> do
         let isRaw =
                 case rsp of
                     ResponseRaw{} -> True
                     _ -> False
+            stCode = statusBS rsp
+            stMsg = msgBS rsp
+        t1 <- getCurrentTime
 
         -- log the status of the response
-        -- this is color coordinated with the request logging
-        -- also includes the request path to connect it to the request
-        unless isRaw $ cb $ mconcat $ map toLogStr $
-            addColor "Status: " ++ statusBS rsp ++
-            [ " "
-            , msgBS rsp
-            , ". "
-            , rawPathInfo req -- if you need help matching the 2 logging statements
-            , "\n"
-            ]
+        cb $ mconcat $ map toLogStr $
+            ansiMethod (requestMethod req) ++ [" ", rawPathInfo req, "\n"] ++
+            params ++
+            ansiColor White "  Accept: " ++ [accept, "\n"] ++
+            if isRaw then [] else
+                ansiColor White "  Status: " ++
+                ansiStatusCode stCode (stCode <> " " <> stMsg) ++
+                [" ", pack $ show $ diffUTCTime t1 t0, "\n"]
+
         sendResponse rsp
   where
-    paramsToBS prefix params =
-      if null params then ""
-        else BS.concat ["\n", prefix, pack (show params)]
-
     allPostParams body =
         case getRequestBodyType req of
             Nothing -> return ([], [])
@@ -275,19 +273,15 @@ detailedMiddleware' cb getAddColor app req sendResponse = do
 
     collectPostParams :: ([Param], [File LBS.ByteString]) -> [Param]
     collectPostParams (postParams, files) = postParams ++
-      map (\(k,v) -> (k, BS.append "FILE: " (fileName v))) files
+      map (\(k,v) -> (k, "FILE: " <> fileName v)) files
 
     readInt bs =
         case reads $ unpack bs of
             (i, _):_ -> Just (i :: Int)
             [] -> Nothing
 
-statusBS :: Response -> [BS.ByteString]
-statusBS rsp =
-    if status >= 400 then ansiColor Red bs else [bs]
-  where
-    bs = pack $ show status
-    status = statusCode $ responseStatus rsp
+statusBS :: Response -> BS.ByteString
+statusBS = pack . show . statusCode . responseStatus
 
 msgBS :: Response -> BS.ByteString
 msgBS = statusMessage . responseStatus
